import React, { useCallback, useMemo, createContext, useContext } from 'react';
import { ApolloClient, InMemoryCache } from '@apollo/client';
import { useAsyncStatuses } from '@onomy/react-utils';
import { OnomyHubConstants } from '@onomy/hub';
import { useAddress, useOnomyHub } from '@onomy/react-hub';
import { OnomyEcosystemConfig } from './OnomyEcosystemConfigs';
import { useExchangePairs } from './exchange/forms/useExchangePairs';
import { OnexHistoryFrames } from './exchange/TheGraph/OnexHistoryFrames';
import { useAsset } from './exchange/hooks';

function useOnomyAppState(ecosystem: OnomyEcosystemConfig) {
  const { wallet } = useOnomyHub();
  const [myOnomyAddress, onomyAddressStatus] = useAddress(OnomyHubConstants.ONOMY_CHAIN);
  const [myOnexAddress, onexAddressStatus] = useAddress(OnomyHubConstants.ONEX_CHAIN);

  const [onomyNomAsset, onomyNomAssetStatus] = useAsset('anom', OnomyHubConstants.ONOMY_CHAIN);
  const [onexNomAsset, onexNomAssetStatus] = useAsset('anom', OnomyHubConstants.ONEX_CHAIN);
  const [onexUsdtAsset, onexUsdtAssetStatus] = useAsset('ausdt', OnomyHubConstants.ONEX_CHAIN);
  const [onexUsdcAsset, onexUsdcAssetStatus] = useAsset('ausdc', OnomyHubConstants.ONEX_CHAIN);
  const [onexEthAsset, onexEthAssetStatus] = useAsset('wei', OnomyHubConstants.ONEX_CHAIN);

  const { loadingStatus: pairsStatus, quoteAssets, pairsByQuote } = useExchangePairs();

  const onexHistoryFrames = useMemo(() => {
    if (!ecosystem.onexGraphqlEndpoint) return null;
    const apolloClient = new ApolloClient({
      uri: ecosystem.onexGraphqlEndpoint,
      cache: new InMemoryCache(),
    });

    return new OnexHistoryFrames(apolloClient);
  }, [ecosystem.onexGraphqlEndpoint]);

  const myOnomyWallet = wallet.onomy.connectedWallet;
  const myOnexWallet = wallet.onex.connectedWallet;

  const loadingStatus = useAsyncStatuses([
    pairsStatus,
    onomyAddressStatus,
    onexAddressStatus,
    onomyNomAssetStatus,
    onexNomAssetStatus,
    onexUsdtAssetStatus,
    onexUsdcAssetStatus,
    onexEthAssetStatus,
  ]);

  const logout = useCallback(async () => {
    await wallet.disconnect();
    window.location.reload();
  }, [wallet]);

  const logoutOnomy = useCallback(async () => {
    await wallet.onomy.disconnect();
    window.location.reload();
  }, [wallet.onomy]);

  const logoutOnex = useCallback(async () => {
    await wallet.onex.disconnect();
    window.location.reload();
  }, [wallet.onex]);

  return {
    ecosystem,

    loadingStatus,

    onexHistoryFrames,

    onomyChain: OnomyHubConstants.ONOMY_CHAIN,
    onexChain: OnomyHubConstants.ONEX_CHAIN,

    onomyNomAsset,
    onexNomAsset,
    onexUsdtAsset,
    onexUsdcAsset,
    onexEthAsset,

    myOnomyAddress,
    myOnexAddress,
    myOnomyWallet,
    myOnexWallet,

    quoteAssets,
    pairsByQuote,

    logout,
    logoutOnomy,
    logoutOnex,
  };
}

export type OnomyAppContextType = ReturnType<typeof useOnomyAppState>;

const OnomyAppContext = createContext<OnomyAppContextType>(null as any);

export function useOnomyApp() {
  const state = useContext(OnomyAppContext);
  if (!state) throw new Error('OnomyAppContext missing');
  return state;
}

export function OnomyAppProvider({
  children,
  ecosystem,
}: {
  children: JSX.Element | JSX.Element[];
  ecosystem: OnomyEcosystemConfig;
}) {
  const state = useOnomyAppState(ecosystem);

  return <OnomyAppContext.Provider value={state}>{children}</OnomyAppContext.Provider>;
}
