import { useMemo } from 'react';
import BigNumber from 'bignumber.js';
import { useAsyncStatuses } from '@onomy/react-utils';
import { DetailedFungibleAssetOnChain, OnomyHubConstants } from '@onomy/hub';
import { useBalanceTotals } from '@onomy/react-hub';
import { useOnexMyDropsForPool, useOnexPoolDetail } from '../hooks';

export function useOnexLiquidity({
  baseAsset,
  quoteAsset,
}: {
  baseAsset: DetailedFungibleAssetOnChain;
  quoteAsset: DetailedFungibleAssetOnChain;
}) {
  const unknownBaseBalance = useMemo(
    () => ({ ...OnomyHubConstants.UNKNOWN_BALANCE, asset: baseAsset }),
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [baseAsset.id, baseAsset.chain.name]
  );
  const unknownQuoteBalance = useMemo(
    () => ({ ...OnomyHubConstants.UNKNOWN_BALANCE, asset: quoteAsset }),
    // eslint-disable-next-line react-hooks/exhaustive-deps
    [quoteAsset.id, baseAsset.chain.name]
  );

  const [pool, poolStatus] = useOnexPoolDetail({ baseAsset, quoteAsset });
  const [myDrops, myDropsStatus] = useOnexMyDropsForPool(pool);

  const basePrincipalAmounts = useMemo(() => myDrops.map(drop => drop.principalBase), [myDrops]);
  const quotePrincipalAmounts = useMemo(() => myDrops.map(drop => drop.principalQuote), [myDrops]);
  const baseRewardAmounts = useMemo(() => myDrops.map(drop => drop.rewardBase), [myDrops]);
  const quoteRewardAmounts = useMemo(() => myDrops.map(drop => drop.rewardQuote), [myDrops]);

  const [[basePrincipal = unknownBaseBalance], basePrincipalStatus] =
    useBalanceTotals(basePrincipalAmounts);
  const [[quotePrincipal = unknownQuoteBalance], quotePrincipalStatus] =
    useBalanceTotals(quotePrincipalAmounts);
  const [[baseReward = unknownBaseBalance], baseRewardStatus] = useBalanceTotals(baseRewardAmounts);
  const [[quoteReward = unknownQuoteBalance], quoteRewardStatus] =
    useBalanceTotals(quoteRewardAmounts);
  const [[baseTotal = unknownBaseBalance], baseTotalStatus] = useBalanceTotals([
    basePrincipal,
    baseReward,
  ]);
  const [[quoteTotal = unknownQuoteBalance], quoteTotalStatus] = useBalanceTotals([
    quotePrincipal,
    quoteReward,
  ]);

  const fiatTotal = baseTotal.fiatValue + quoteTotal.fiatValue;

  const totalsStatus = useAsyncStatuses([
    basePrincipalStatus,
    quotePrincipalStatus,
    baseRewardStatus,
    quoteRewardStatus,
    baseTotalStatus,
    quoteTotalStatus,
  ]);

  const totalRewardRatio = useMemo(() => {
    const bnPrinciple = new BigNumber(basePrincipal.amount.toString());
    const bnReward = new BigNumber(baseReward.amount.toString());
    const bnTotal = bnReward.plus(bnPrinciple);
    return bnReward.dividedBy(bnTotal).toNumber();
  }, [basePrincipal.amount, baseReward.amount]);

  const loadingStatus = useAsyncStatuses([poolStatus, myDropsStatus, totalsStatus]);

  return {
    loadingStatus,
    poolStatus,
    myDropsStatus,
    basePrincipalStatus,
    quotePrincipalStatus,
    baseRewardStatus,
    quoteRewardStatus,
    totalsStatus,

    pool,
    myDrops,

    basePrincipal,
    quotePrincipal,
    baseReward,
    quoteReward,
    baseTotal,
    quoteTotal,
    fiatTotal,

    totalRewardRatio,
  };
}
